@@ -26,6 +26,7 @@
 #include "projectwindow.h"
 
 #include "buildinfo.h"
+#include "projectexplorerconstants.h"
 #include "kit.h"
 #include "kitmanager.h"
 #include "kitoptionspage.h"
@@ -38,18 +39,28 @@
 #include "target.h"
 #include "targetsettingspanel.h"
 
+#include <coreplugin/actionmanager/actionmanager.h>
+#include <coreplugin/actionmanager/commandbutton.h>
+#include <coreplugin/coreconstants.h>
 #include <coreplugin/coreicons.h>
+#include <coreplugin/find/optionspopup.h>
+#include <coreplugin/findplaceholder.h>
+#include <coreplugin/icontext.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/idocument.h>
-
-#include <extensionsystem/pluginmanager.h>
+#include <coreplugin/outputwindow.h>
 
 #include <utils/algorithm.h>
 #include <utils/basetreeview.h>
+#include <utils/hostosinfo.h>
 #include <utils/navigationtreeview.h>
 #include <utils/qtcassert.h>
 #include <utils/styledbar.h>
 #include <utils/treemodel.h>
+#include <utils/utilsicons.h>
+
+#include <texteditor/fontsettings.h>
+#include <texteditor/texteditorsettings.h>
 
 #include <QApplication>
 #include <QComboBox>
@@ -72,6 +83,151 @@ namespace Internal {
 
 class MiscSettingsGroupItem;
 
+const char kBuildSystemOutputContext[] = "ProjectsMode.BuildSystemOutput";
+const char kRegExpActionId[] = "OutputFilter.RegularExpressions.BuildSystemOutput";
+const char kCaseSensitiveActionId[] = "OutputFilter.CaseSensitive.BuildSystemOutput";
+const char kInvertActionId[] = "OutputFilter.Invert.BuildSystemOutput";
+
+class BuildSystemOutputWindow : public OutputWindow
+{
+public:
+    BuildSystemOutputWindow();
+
+    QWidget *toolBar();
+
+private:
+    void updateFilter();
+
+    QPointer<QWidget> m_toolBar;
+    QPointer<FancyLineEdit> m_filterOutputLineEdit;
+    QAction *m_clear;
+    QAction *m_filterActionRegexp;
+    QAction *m_filterActionCaseSensitive;
+    QAction *m_invertFilterAction;
+    QAction *m_zoomIn;
+    QAction *m_zoomOut;
+};
+
+BuildSystemOutputWindow::BuildSystemOutputWindow()
+    : OutputWindow(Context(kBuildSystemOutputContext), "ProjectsMode.BuildSystemOutput.Zoom")
+    , m_clear(new QAction)
+{
+    setReadOnly(true);
+
+    Command *clearCommand = ActionManager::command(Core::Constants::OUTPUTPANE_CLEAR);
+    m_clear->setIcon(Utils::Icons::CLEAN_TOOLBAR.icon());
+    m_clear->setText(clearCommand->action()->text());
+    ActionManager::registerAction(m_clear,
+                                  Core::Constants::OUTPUTPANE_CLEAR,
+                                  Context(kBuildSystemOutputContext));
+    connect(m_clear, &QAction::triggered, this, [this] { clear(); });
+
+    m_filterActionRegexp = new QAction(this);
+    m_filterActionRegexp->setCheckable(true);
+    m_filterActionRegexp->setText(ProjectWindow::tr("Use Regular Expressions"));
+    connect(m_filterActionRegexp, &QAction::toggled, this, &BuildSystemOutputWindow::updateFilter);
+    Core::ActionManager::registerAction(m_filterActionRegexp,
+                                        kRegExpActionId,
+                                        Context(Constants::C_PROJECTEXPLORER));
+
+    m_filterActionCaseSensitive = new QAction(this);
+    m_filterActionCaseSensitive->setCheckable(true);
+    m_filterActionCaseSensitive->setText(ProjectWindow::tr("Case Sensitive"));
+    connect(m_filterActionCaseSensitive,
+            &QAction::toggled,
+            this,
+            &BuildSystemOutputWindow::updateFilter);
+    Core::ActionManager::registerAction(m_filterActionCaseSensitive,
+                                        kCaseSensitiveActionId,
+                                        Context(Constants::C_PROJECTEXPLORER));
+
+    m_invertFilterAction = new QAction(this);
+    m_invertFilterAction->setCheckable(true);
+    m_invertFilterAction->setText(ProjectWindow::tr("Show Non-matching Lines"));
+    connect(m_invertFilterAction, &QAction::toggled, this, &BuildSystemOutputWindow::updateFilter);
+    Core::ActionManager::registerAction(m_invertFilterAction,
+                                        kInvertActionId,
+                                        Context(Constants::C_PROJECTEXPLORER));
+
+    connect(TextEditor::TextEditorSettings::instance(),
+            &TextEditor::TextEditorSettings::fontSettingsChanged,
+            this,
+            [this] { setBaseFont(TextEditor::TextEditorSettings::fontSettings().font()); });
+    setBaseFont(TextEditor::TextEditorSettings::fontSettings().font());
+
+    m_zoomIn = new QAction;
+    m_zoomIn->setIcon(Utils::Icons::PLUS_TOOLBAR.icon());
+    connect(m_zoomIn, &QAction::triggered, this, [this] { zoomIn(); });
+    ActionManager::registerAction(m_zoomIn,
+                                  Core::Constants::ZOOM_IN,
+                                  Context(kBuildSystemOutputContext));
+
+    m_zoomOut = new QAction;
+    m_zoomOut->setIcon(Utils::Icons::MINUS.icon());
+    connect(m_zoomOut, &QAction::triggered, this, [this] { zoomOut(); });
+    ActionManager::registerAction(m_zoomOut,
+                                  Core::Constants::ZOOM_OUT,
+                                  Context(kBuildSystemOutputContext));
+}
+
+QWidget *BuildSystemOutputWindow::toolBar()
+{
+    if (!m_toolBar) {
+        m_toolBar = new StyledBar(this);
+        auto clearButton = new CommandButton(Core::Constants::OUTPUTPANE_CLEAR);
+        clearButton->setDefaultAction(m_clear);
+        clearButton->setToolTipBase(m_clear->text());
+
+        m_filterOutputLineEdit = new FancyLineEdit;
+        m_filterOutputLineEdit->setButtonVisible(FancyLineEdit::Left, true);
+        m_filterOutputLineEdit->setButtonIcon(FancyLineEdit::Left, Utils::Icons::MAGNIFIER.icon());
+        m_filterOutputLineEdit->setFiltering(true);
+        m_filterOutputLineEdit->setHistoryCompleter("ProjectsMode.BuildSystemOutput.Filter");
+        connect(m_filterOutputLineEdit,
+                &FancyLineEdit::textChanged,
+                this,
+                &BuildSystemOutputWindow::updateFilter);
+        connect(m_filterOutputLineEdit,
+                &FancyLineEdit::returnPressed,
+                this,
+                &BuildSystemOutputWindow::updateFilter);
+        connect(m_filterOutputLineEdit, &FancyLineEdit::leftButtonClicked, this, [this] {
+            auto popup = new Core::OptionsPopup(m_filterOutputLineEdit,
+                                                {kRegExpActionId,
+                                                 kCaseSensitiveActionId,
+                                                 kInvertActionId});
+            popup->show();
+        });
+
+        auto zoomInButton = new CommandButton(Core::Constants::ZOOM_IN);
+        zoomInButton->setDefaultAction(m_zoomIn);
+        auto zoomOutButton = new CommandButton(Core::Constants::ZOOM_OUT);
+        zoomOutButton->setDefaultAction(m_zoomOut);
+
+        auto layout = new QHBoxLayout;
+        layout->setContentsMargins(0, 0, 0, 0);
+        layout->setSpacing(0);
+        m_toolBar->setLayout(layout);
+        layout->addWidget(clearButton);
+        layout->addWidget(m_filterOutputLineEdit);
+        layout->addWidget(zoomInButton);
+        layout->addWidget(zoomOutButton);
+        layout->addStretch();
+    }
+    return m_toolBar;
+}
+
+void BuildSystemOutputWindow::updateFilter()
+{
+    if (!m_filterOutputLineEdit)
+        return;
+    updateFilterProperties(m_filterOutputLineEdit->text(),
+                           m_filterActionCaseSensitive->isChecked() ? Qt::CaseSensitive
+                                                                    : Qt::CaseInsensitive,
+                           m_filterActionRegexp->isChecked(),
+                           m_invertFilterAction->isChecked());
+}
+
 // Standard third level for the generic case: i.e. all except for the Build/Run page
 class MiscSettingsPanelItem : public TreeItem // TypedTreeItem<TreeItem, MiscSettingsGroupItem>
 {
@@ -80,12 +236,14 @@ public:
         : m_factory(factory), m_project(project)
     {}
 
-    ~MiscSettingsPanelItem() { delete m_widget; }
+    ~MiscSettingsPanelItem() override { delete m_widget; }
 
     QVariant data(int column, int role) const override;
     Qt::ItemFlags flags(int column) const override;
     bool setData(int column, const QVariant &, int role) override;
 
+    ProjectPanelFactory *factory() const { return m_factory; }
+
 protected:
     ProjectPanelFactory *m_factory = nullptr;
     QPointer<Project> m_project;
@@ -104,9 +262,7 @@ QVariant MiscSettingsPanelItem::data(int column, int role) const
     if (role == PanelWidgetRole) {
         if (!m_widget) {
             QWidget *widget = m_factory->createWidget(m_project);
-            m_widget = new PanelsWidget(m_factory->displayName(),
-                                        QIcon(m_factory->icon()),
-                                        widget);
+            m_widget = new PanelsWidget(m_factory->displayName(), widget);
             m_widget->setFocusProxy(widget);
         }
 
@@ -176,7 +332,7 @@ public:
         Q_UNUSED(column)
 
         if (role == ItemActivatedFromBelowRole) {
-            TreeItem *item = data.value<TreeItem *>();
+            auto *item = data.value<TreeItem *>();
             QTC_ASSERT(item, return false);
             m_currentPanelIndex = indexOf(item);
             QTC_ASSERT(m_currentPanelIndex != -1, return false);
@@ -200,7 +356,7 @@ private:
 class ProjectItem : public TreeItem
 {
 public:
-    ProjectItem() {}
+    ProjectItem() = default;
 
     ProjectItem(Project *project, const std::function<void()> &changeListener)
         : m_project(project), m_changeListener(changeListener)
@@ -215,8 +371,6 @@ public:
     {
         switch (role) {
         case Qt::DisplayRole:
-            return m_project->displayName();
-
         case ProjectDisplayNameRole:
             return m_project->displayName();
 
@@ -281,10 +435,17 @@ public:
 
     QModelIndex activeIndex() const
     {
-        TreeItem *activeItem = data(0, ActiveItemRole).value<TreeItem *>();
+        auto *activeItem = data(0, ActiveItemRole).value<TreeItem *>();
         return activeItem ? activeItem->index() : QModelIndex();
     }
 
+    TreeItem *itemForProjectPanel(Utils::Id panelId)
+    {
+        return m_miscItem->findChildAtLevel(1, [panelId](const TreeItem *item){
+            return static_cast<const MiscSettingsPanelItem *>(item)->factory()->id() == panelId;
+        });
+    }
+
 private:
     int m_currentChildIndex = 0; // Start with Build & Run.
     Project *m_project = nullptr;
@@ -297,7 +458,7 @@ private:
 class SelectorDelegate : public QStyledItemDelegate
 {
 public:
-    SelectorDelegate() {}
+    SelectorDelegate() = default;
 
     QSize sizeHint(const QStyleOptionViewItem &option,
                    const QModelIndex &index) const final;
@@ -332,11 +493,31 @@ public:
         setContextMenuPolicy(Qt::CustomContextMenu);
     }
 
+private:
     // remove branch indicators
     void drawBranches(QPainter *, const QRect &, const QModelIndex &) const final
     {
         return;
     }
+
+    bool userWantsContextMenu(const QMouseEvent *e) const
+    {
+        // On Windows, we get additional mouse events for the item view when right-clicking,
+        // causing unwanted kit activation (QTCREATORBUG-24156). Let's suppress these.
+        return HostOsInfo::isWindowsHost() && e->button() == Qt::RightButton;
+    }
+
+    void mousePressEvent(QMouseEvent *e) final
+    {
+        if (!userWantsContextMenu(e))
+            BaseTreeView::mousePressEvent(e);
+    }
+
+    void mouseReleaseEvent(QMouseEvent *e) final
+    {
+        if (!userWantsContextMenu(e))
+            BaseTreeView::mouseReleaseEvent(e);
+    }
 };
 
 class ComboBoxItem : public TreeItem
@@ -369,7 +550,7 @@ public:
 
         m_selectorTree = new SelectorTree;
         m_selectorTree->setModel(&m_projectsModel);
-        m_selectorTree->setItemDelegate(new SelectorDelegate);
+        m_selectorTree->setItemDelegate(&m_selectorDelegate);
         m_selectorTree->setContextMenuPolicy(Qt::CustomContextMenu);
         connect(m_selectorTree, &QAbstractItemView::activated,
                 this, &ProjectWindowPrivate::itemActivated);
@@ -378,9 +559,17 @@ public:
 
         m_projectSelection = new QComboBox;
         m_projectSelection->setModel(&m_comboBoxModel);
-        connect(m_projectSelection, static_cast<void(QComboBox::*)(int)>(&QComboBox::activated),
+        connect(m_projectSelection, QOverload<int>::of(&QComboBox::activated),
                 this, &ProjectWindowPrivate::projectSelected, Qt::QueuedConnection);
 
+        const auto switchProjectAction = new QAction(this);
+        ActionManager::registerAction(switchProjectAction, Core::Constants::GOTOPREVINHISTORY,
+                                      Context(Constants::C_PROJECTEXPLORER));
+        connect(switchProjectAction, &QAction::triggered, this, [this] {
+            if (m_projectSelection->count() > 1)
+                m_projectSelection->showPopup();
+        });
+
         SessionManager *sessionManager = SessionManager::instance();
         connect(sessionManager, &SessionManager::projectAdded,
                 this, &ProjectWindowPrivate::registerProject);
@@ -419,12 +608,13 @@ public:
 
         auto innerLayout = new QVBoxLayout;
         innerLayout->setSpacing(10);
-        innerLayout->setContentsMargins(14, innerLayout->spacing(), 14, 0);
+        innerLayout->setContentsMargins(PanelsWidget::PanelVMargin, innerLayout->spacing(),
+                                        PanelsWidget::PanelVMargin, 0);
         innerLayout->addWidget(m_manageKits);
-        innerLayout->addWidget(m_importBuild);
         innerLayout->addSpacerItem(new QSpacerItem(10, 30, QSizePolicy::Maximum, QSizePolicy::Maximum));
         innerLayout->addWidget(activeLabel);
         innerLayout->addWidget(m_projectSelection);
+        innerLayout->addWidget(m_importBuild);
         innerLayout->addWidget(m_selectorTree);
 
         auto selectorLayout = new QVBoxLayout(selectorView);
@@ -434,11 +624,30 @@ public:
 
         auto selectorDock = q->addDockForWidget(selectorView, true);
         q->addDockWidget(Qt::LeftDockWidgetArea, selectorDock);
+
+        m_buildSystemOutput = new BuildSystemOutputWindow;
+        auto output = new QWidget;
+        // ProjectWindow sets background role to Base which is wrong for the output window,
+        // especially the find tool bar (resulting in wrong label color)
+        output->setBackgroundRole(QPalette::Window);
+        output->setObjectName("BuildSystemOutput");
+        output->setWindowTitle(ProjectWindow::tr("Build System Output"));
+        auto outputLayout = new QVBoxLayout;
+        output->setLayout(outputLayout);
+        outputLayout->setContentsMargins(0, 0, 0, 0);
+        outputLayout->setSpacing(0);
+        outputLayout->addWidget(m_buildSystemOutput->toolBar());
+        outputLayout->addWidget(m_buildSystemOutput);
+        outputLayout->addWidget(new FindToolBarPlaceHolder(m_buildSystemOutput));
+        auto outputDock = q->addDockForWidget(output, true);
+        q->addDockWidget(Qt::RightDockWidgetArea, outputDock);
     }
 
     void updatePanel()
     {
         ProjectItem *projectItem = m_projectsModel.rootItem()->childAt(0);
+        if (!projectItem)
+            return;
         setPanel(projectItem->data(0, PanelWidgetRole).value<QWidget *>());
 
         QModelIndex activeIndex = projectItem->activeIndex();
@@ -499,6 +708,14 @@ public:
             item->setData(0, QVariant(), ItemActivatedDirectlyRole);
     }
 
+    void activateProjectPanel(Utils::Id panelId)
+    {
+        if (ProjectItem *projectItem = m_projectsModel.rootItem()->childAt(0)) {
+            if (TreeItem *item = projectItem->itemForProjectPanel(panelId))
+                itemActivated(item->index());
+        }
+    }
+
     void openContextMenu(const QPoint &pos)
     {
         QMenu menu;
@@ -529,38 +746,32 @@ public:
     void handleManageKits()
     {
         if (ProjectItem *projectItem = m_projectsModel.rootItem()->childAt(0)) {
-            if (KitOptionsPage *page = ExtensionSystem::PluginManager::getObject<KitOptionsPage>())
-                page->showKit(KitManager::kit(Id::fromSetting(projectItem->data(0, KitIdRole))));
+            if (auto kitPage = KitOptionsPage::instance())
+                kitPage->showKit(KitManager::kit(Id::fromSetting(projectItem->data(0, KitIdRole))));
         }
-        ICore::showOptionsDialog(Constants::KITS_SETTINGS_PAGE_ID, ICore::mainWindow());
+        ICore::showOptionsDialog(Constants::KITS_SETTINGS_PAGE_ID);
     }
 
     void handleImportBuild()
     {
         ProjectItem *projectItem = m_projectsModel.rootItem()->childAt(0);
-        Project *project = projectItem ? projectItem->project() : 0;
-        ProjectImporter *projectImporter = project ? project->projectImporter() : 0;
+        Project *project = projectItem ? projectItem->project() : nullptr;
+        ProjectImporter *projectImporter = project ? project->projectImporter() : nullptr;
         QTC_ASSERT(projectImporter, return);
 
-        QString dir = project->projectDirectory().toString();
-        QString importDir = QFileDialog::getExistingDirectory(ICore::mainWindow(),
-                                                              ProjectWindow::tr("Import Directory"),
-                                                              dir);
-        FileName path = FileName::fromString(importDir);
+        FilePath importDir =
+                FileUtils::getExistingDirectory(nullptr, ProjectWindow::tr("Import Directory"),
+                                                project->projectDirectory());
 
         Target *lastTarget = nullptr;
         BuildConfiguration *lastBc = nullptr;
-        const QList<BuildInfo *> toImport = projectImporter->import(path, false);
-        for (BuildInfo *info : toImport) {
-            Target *target = project->target(info->kitId);
-            if (!target) {
-                target = project->createTarget(KitManager::kit(info->kitId));
-                if (target)
-                    project->addTarget(target);
-            }
+        for (const BuildInfo &info : projectImporter->import(importDir, false)) {
+            Target *target = project->target(info.kitId);
+            if (!target)
+                target = project->addTargetForKit(KitManager::kit(info.kitId));
             if (target) {
                 projectImporter->makePersistent(target->kit());
-                BuildConfiguration *bc = info->factory()->create(target, info);
+                BuildConfiguration *bc = info.factory->create(target, info);
                 QTC_ASSERT(bc, continue);
                 target->addBuildConfiguration(bc);
 
@@ -572,12 +783,11 @@ public:
             SessionManager::setActiveBuildConfiguration(lastTarget, lastBc, SetActive::Cascade);
             SessionManager::setActiveTarget(project, lastTarget, SetActive::Cascade);
         }
-
-        qDeleteAll(toImport);
     }
 
     void setPanel(QWidget *panel)
     {
+        q->savePersistentSettings();
         if (QWidget *widget = q->centralWidget()) {
             q->takeCentralWidget();
             widget->hide(); // Don't delete.
@@ -588,15 +798,18 @@ public:
             if (q->hasFocus()) // we get assigned focus from setFocusToCurrentMode, pass that on
                 panel->setFocus();
         }
+        q->loadPersistentSettings();
     }
 
     ProjectWindow *q;
     ProjectsModel m_projectsModel;
     ComboBoxModel m_comboBoxModel;
+    SelectorDelegate m_selectorDelegate;
     QComboBox *m_projectSelection;
     SelectorTree *m_selectorTree;
     QPushButton *m_importBuild;
     QPushButton *m_manageKits;
+    BuildSystemOutputWindow *m_buildSystemOutput;
 };
 
 //
@@ -604,7 +817,7 @@ public:
 //
 
 ProjectWindow::ProjectWindow()
-    : d(new ProjectWindowPrivate(this))
+    : d(std::make_unique<ProjectWindowPrivate>(this))
 {
     setBackgroundRole(QPalette::Base);
 
@@ -613,9 +826,50 @@ ProjectWindow::ProjectWindow()
     setContextMenuPolicy(Qt::CustomContextMenu);
 }
 
-ProjectWindow::~ProjectWindow()
+void ProjectWindow::activateProjectPanel(Utils::Id panelId)
 {
-    delete d;
+    d->activateProjectPanel(panelId);
+}
+
+OutputWindow *ProjectWindow::buildSystemOutput() const
+{
+    return d->m_buildSystemOutput;
+}
+
+void ProjectWindow::hideEvent(QHideEvent *event)
+{
+    savePersistentSettings();
+    FancyMainWindow::hideEvent(event);
+}
+
+void ProjectWindow::showEvent(QShowEvent *event)
+{
+    FancyMainWindow::showEvent(event);
+    loadPersistentSettings();
+}
+
+ProjectWindow::~ProjectWindow() = default;
+
+const char PROJECT_WINDOW_KEY[] = "ProjectExplorer.ProjectWindow";
+
+void ProjectWindow::savePersistentSettings() const
+{
+    if (!centralWidget())
+        return;
+    QSettings * const settings = ICore::settings();
+    settings->beginGroup(PROJECT_WINDOW_KEY);
+    saveSettings(settings);
+    settings->endGroup();
+}
+
+void ProjectWindow::loadPersistentSettings()
+{
+    if (!centralWidget())
+        return;
+    QSettings * const settings = ICore::settings();
+    settings->beginGroup(PROJECT_WINDOW_KEY);
+    restoreSettings(settings);
+    settings->endGroup();
 }
 
 QSize SelectorDelegate::sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
