@@ -187,9 +187,9 @@ public:
     { }
 
 protected:
-    QString trackFile(const QString &repository) override;
+    FilePath trackFile(const FilePath &repository) override;
 
-    QString refreshTopic(const QString &repository) override;
+    QString refreshTopic(const FilePath &repository) override;
 
 private:
     SubversionPluginPrivate *m_plugin;
@@ -206,21 +206,21 @@ public:
     // IVersionControl
     QString displayName() const final;
     Utils::Id id() const final;
-    bool isVcsFileOrDirectory(const Utils::FilePath &fileName) const final;
+    bool isVcsFileOrDirectory(const FilePath &filePath) const final;
 
-    bool managesDirectory(const QString &directory, QString *topLevel) const final;
-    bool managesFile(const QString &workingDirectory, const QString &fileName) const final;
+    bool managesDirectory(const FilePath &directory, FilePath *topLevel) const final;
+    bool managesFile(const FilePath &workingDirectory, const QString &fileName) const final;
 
     bool isConfigured() const final;
     bool supportsOperation(Operation operation) const final;
-    bool vcsOpen(const QString &fileName) final;
-    bool vcsAdd(const QString &fileName) final;
-    bool vcsDelete(const QString &filename) final;
-    bool vcsMove(const QString &from, const QString &to) final;
-    bool vcsCreateRepository(const QString &directory) final;
+    bool vcsOpen(const FilePath &filePath) final;
+    bool vcsAdd(const FilePath &filePath) final;
+    bool vcsDelete(const FilePath &filePath) final;
+    bool vcsMove(const FilePath &from, const FilePath &to) final;
+    bool vcsCreateRepository(const FilePath &directory) final;
 
-    void vcsAnnotate(const QString &file, int line) final;
-    void vcsDescribe(const QString &source, const QString &changeNr) final;
+    void vcsAnnotate(const FilePath &file, int line) final;
+    void vcsDescribe(const FilePath &source, const QString &changeNr) final;
 
     Core::ShellCommand *createInitialCheckoutCommand(const QString &url,
                                                      const Utils::FilePath &baseDirectory,
@@ -235,20 +235,20 @@ public:
     SubversionSubmitEditor *openSubversionSubmitEditor(const QString &fileName);
 
     // IVersionControl
-    bool vcsAdd(const QString &workingDir, const QString &fileName);
-    bool vcsDelete(const QString &workingDir, const QString &fileName);
-    bool vcsMove(const QString &workingDir, const QString &from, const QString &to);
-    bool vcsCheckout(const QString &directory, const QByteArray &url);
+    bool vcsAdd(const FilePath &workingDir, const QString &fileName);
+    bool vcsDelete(const FilePath &workingDir, const QString &fileName);
+    bool vcsMove(const FilePath &workingDir, const QString &from, const QString &to);
+    bool vcsCheckout(const FilePath &directory, const QByteArray &url);
 
     static SubversionPluginPrivate *instance();
 
-    QString monitorFile(const QString &repository) const;
-    QString synchronousTopic(const QString &repository) const;
-    SubversionResponse runSvn(const QString &workingDir,
+    QString monitorFile(const FilePath &repository) const;
+    QString synchronousTopic(const FilePath &repository) const;
+    SubversionResponse runSvn(const FilePath &workingDir,
                               const QStringList &arguments, int timeOutS,
                               unsigned flags, QTextCodec *outputCodec = nullptr) const;
-    void vcsAnnotateHelper(const QString &workingDir, const QString &file,
-                     const QString &revision = QString(), int lineNumber = -1);
+    void vcsAnnotateHelper(const FilePath &workingDir, const QString &file,
+                           const QString &revision = {}, int lineNumber = -1);
 
 protected:
     void updateActions(VcsBase::VcsBasePluginPrivate::ActionState) override;
@@ -282,20 +282,20 @@ private:
                                       Utils::Id id, const QString &source,
                                       QTextCodec *codec);
 
-    void filelog(const QString &workingDir,
-                 const QString &file = QString(),
+    void filelog(const FilePath &workingDir,
+                 const QString &file = {},
                  bool enableAnnotationContextMenu = false);
-    void svnStatus(const QString &workingDir, const QString &relativePath = QString());
-    void svnUpdate(const QString &workingDir, const QString &relativePath = QString());
+    void svnStatus(const FilePath &workingDir, const QString &relativePath = {});
+    void svnUpdate(const FilePath &workingDir, const QString &relativePath = {});
     bool checkSVNSubDir(const QDir &directory) const;
-    void startCommit(const QString &workingDir, const QStringList &files = QStringList());
+    void startCommit(const FilePath &workingDir, const QStringList &files = {});
 
     const QStringList m_svnDirectories;
 
     SubversionSettings m_settings;
     SubversionClient *m_client = nullptr;
     QString m_commitMessageFileName;
-    QString m_commitRepository;
+    FilePath m_commitRepository;
 
     Core::CommandLocator *m_commandLocator = nullptr;
     Utils::ParameterAction *m_addAction = nullptr;
@@ -624,7 +624,8 @@ void SubversionPluginPrivate::diffCommitFiles(const QStringList &files)
 
 SubversionSubmitEditor *SubversionPluginPrivate::openSubversionSubmitEditor(const QString &fileName)
 {
-    IEditor *editor = EditorManager::openEditor(fileName, Constants::SUBVERSION_COMMIT_EDITOR_ID);
+    IEditor *editor = EditorManager::openEditor(FilePath::fromString(fileName),
+                                                Constants::SUBVERSION_COMMIT_EDITOR_ID);
     auto submitEditor = qobject_cast<SubversionSubmitEditor*>(editor);
     QTC_ASSERT(submitEditor, return nullptr);
     setSubmitEditor(submitEditor);
@@ -690,7 +691,7 @@ void SubversionPluginPrivate::revertAll()
     QStringList args;
     args << QLatin1String("revert");
     args << SubversionClient::addAuthenticationOptions(m_settings);
-    args << QLatin1String("--recursive") << state.topLevel();
+    args << QLatin1String("--recursive") << state.topLevel().toString();
     const SubversionResponse revertResponse
             = runSvn(state.topLevel(), args, m_settings.timeout.value(),
                      VcsCommand::SshPasswordPrompt | VcsCommand::ShowStdOut);
@@ -781,7 +782,7 @@ void SubversionPluginPrivate::startCommitProject()
 /* Start commit of files of a single repository by displaying
  * template and files in a submit editor. On closing, the real
  * commit will start. */
-void SubversionPluginPrivate::startCommit(const QString &workingDir, const QStringList &files)
+void SubversionPluginPrivate::startCommit(const FilePath &workingDir, const QStringList &files)
 {
     if (!promptBeforeCommit())
         return;
@@ -869,7 +870,7 @@ void SubversionPluginPrivate::updateRepository()
     svnUpdate(state.topLevel());
 }
 
-void SubversionPluginPrivate::svnStatus(const QString &workingDir, const QString &relativePath)
+void SubversionPluginPrivate::svnStatus(const FilePath &workingDir, const QString &relativePath)
 {
     const VcsBasePluginState state = currentState();
     QTC_ASSERT(state.hasTopLevel(), return);
@@ -877,15 +878,15 @@ void SubversionPluginPrivate::svnStatus(const QString &workingDir, const QString
     args << SubversionClient::addAuthenticationOptions(m_settings);
     if (!relativePath.isEmpty())
         args.append(SubversionClient::escapeFile(relativePath));
-    VcsOutputWindow::setRepository(workingDir);
+    VcsOutputWindow::setRepository(workingDir.toString());
     runSvn(workingDir, args, m_settings.timeout.value(),
            VcsCommand::ShowStdOut | VcsCommand::ShowSuccessMessage);
     VcsOutputWindow::clearRepository();
 }
 
-void SubversionPluginPrivate::filelog(const QString &workingDir,
-                               const QString &file,
-                               bool enableAnnotationContextMenu)
+void SubversionPluginPrivate::filelog(const FilePath &workingDir,
+                                      const QString &file,
+                                      bool enableAnnotationContextMenu)
 {
     m_client->log(workingDir, QStringList(file), QStringList(), enableAnnotationContextMenu);
 }
@@ -897,7 +898,7 @@ void SubversionPluginPrivate::updateProject()
     svnUpdate(state.currentProjectTopLevel(), state.relativeCurrentProject());
 }
 
-void SubversionPluginPrivate::svnUpdate(const QString &workingDir, const QString &relativePath)
+void SubversionPluginPrivate::svnUpdate(const FilePath &workingDir, const QString &relativePath)
 {
     QStringList args(QLatin1String("update"));
     args << SubversionClient::addAuthenticationOptions(m_settings);
@@ -918,7 +919,7 @@ void SubversionPluginPrivate::annotateCurrentFile()
     vcsAnnotateHelper(state.currentFileTopLevel(), state.relativeCurrentFile());
 }
 
-void SubversionPluginPrivate::vcsAnnotateHelper(const QString &workingDir, const QString &file,
+void SubversionPluginPrivate::vcsAnnotateHelper(const FilePath &workingDir, const QString &file,
                                                 const QString &revision /* = QString() */,
                                                 int lineNumber /* = -1 */)
 {
@@ -947,7 +948,7 @@ void SubversionPluginPrivate::vcsAnnotateHelper(const QString &workingDir, const
     // Determine id
     const QStringList files = QStringList(file);
     const QString id = VcsBaseEditor::getTitleId(workingDir, files, revision);
-    const QString tag = VcsBaseEditor::editorTag(AnnotateOutput, workingDir, files);
+    const QString tag = VcsBaseEditor::editorTag(AnnotateOutput, workingDir.toString(), files);
     if (IEditor *editor = VcsBaseEditor::locateEditorByTag(tag)) {
         editor->document()->setContents(response.stdOut.toUtf8());
         VcsBaseEditor::gotoLineOfEditor(editor, lineNumber);
@@ -967,13 +968,13 @@ void SubversionPluginPrivate::projectStatus()
     svnStatus(state.currentProjectTopLevel(), state.relativeCurrentProject());
 }
 
-void SubversionPluginPrivate::vcsDescribe(const QString &source, const QString &changeNr)
+void SubversionPluginPrivate::vcsDescribe(const FilePath &source, const QString &changeNr)
 {
     // To describe a complete change, find the top level and then do
     //svn diff -r 472958:472959 <top level>
-    const QFileInfo fi(source);
-    QString topLevel;
-    const bool manages = managesDirectory(fi.isDir() ? source : fi.absolutePath(), &topLevel);
+    const QFileInfo fi = source.toFileInfo();
+    FilePath topLevel;
+    const bool manages = managesDirectory(fi.isDir() ? source : FilePath::fromString(fi.absolutePath()), &topLevel);
     if (!manages || topLevel.isEmpty())
         return;
     if (Subversion::Constants::debug)
@@ -1014,10 +1015,10 @@ void SubversionPluginPrivate::commitFromEditor()
     EditorManager::closeDocuments({submitEditor()->document()});
 }
 
-SubversionResponse SubversionPluginPrivate::runSvn(const QString &workingDir,
-                                            const QStringList &arguments,
-                                            int timeOutS, unsigned flags,
-                                            QTextCodec *outputCodec) const
+SubversionResponse SubversionPluginPrivate::runSvn(const FilePath &workingDir,
+                                                   const QStringList &arguments,
+                                                   int timeOutS, unsigned flags,
+                                                   QTextCodec *outputCodec) const
 {
     SubversionResponse response;
     if (m_settings.binaryPath.value().isEmpty()) {
@@ -1067,10 +1068,10 @@ SubversionPluginPrivate *SubversionPluginPrivate::instance()
     return dd;
 }
 
-QString SubversionPluginPrivate::monitorFile(const QString &repository) const
+QString SubversionPluginPrivate::monitorFile(const FilePath &repository) const
 {
     QTC_ASSERT(!repository.isEmpty(), return QString());
-    QDir repoDir(repository);
+    QDir repoDir(repository.toString());
     foreach (const QString &svnDir, m_svnDirectories) {
         if (repoDir.exists(svnDir)) {
             QFileInfo fi(repoDir.absoluteFilePath(svnDir + QLatin1String("/wc.db")));
@@ -1081,12 +1082,12 @@ QString SubversionPluginPrivate::monitorFile(const QString &repository) const
     return QString();
 }
 
-QString SubversionPluginPrivate::synchronousTopic(const QString &repository) const
+QString SubversionPluginPrivate::synchronousTopic(const FilePath &repository) const
 {
     return m_client->synchronousTopic(repository);
 }
 
-bool SubversionPluginPrivate::vcsAdd(const QString &workingDir, const QString &rawFileName)
+bool SubversionPluginPrivate::vcsAdd(const FilePath &workingDir, const QString &rawFileName)
 {
     const QString file = QDir::toNativeSeparators(SubversionClient::escapeFile(rawFileName));
     QStringList args;
@@ -1099,7 +1100,7 @@ bool SubversionPluginPrivate::vcsAdd(const QString &workingDir, const QString &r
     return !response.error;
 }
 
-bool SubversionPluginPrivate::vcsDelete(const QString &workingDir, const QString &rawFileName)
+bool SubversionPluginPrivate::vcsDelete(const FilePath &workingDir, const QString &rawFileName)
 {
     const QString file = QDir::toNativeSeparators(SubversionClient::escapeFile(rawFileName));
 
@@ -1114,7 +1115,7 @@ bool SubversionPluginPrivate::vcsDelete(const QString &workingDir, const QString
     return !response.error;
 }
 
-bool SubversionPluginPrivate::vcsMove(const QString &workingDir, const QString &from, const QString &to)
+bool SubversionPluginPrivate::vcsMove(const FilePath &workingDir, const QString &from, const QString &to)
 {
     QStringList args(QLatin1String("move"));
     args << SubversionClient::addAuthenticationOptions(m_settings);
@@ -1127,7 +1128,7 @@ bool SubversionPluginPrivate::vcsMove(const QString &workingDir, const QString &
     return !response.error;
 }
 
-bool SubversionPluginPrivate::vcsCheckout(const QString &directory, const QByteArray &url)
+bool SubversionPluginPrivate::vcsCheckout(const FilePath &directory, const QByteArray &url)
 {
     QUrl tempUrl = QUrl::fromEncoded(url);
     QString username = tempUrl.userName();
@@ -1147,7 +1148,7 @@ bool SubversionPluginPrivate::vcsCheckout(const QString &directory, const QByteA
             args << QLatin1String("--password") << password;
     }
 
-    args << QLatin1String(tempUrl.toEncoded()) << directory;
+    args << QLatin1String(tempUrl.toEncoded()) << directory.toString();
 
     const SubversionResponse response
             = runSvn(directory, args, 10 * m_settings.timeout.value(), VcsCommand::SshPasswordPrompt);
@@ -1155,9 +1156,9 @@ bool SubversionPluginPrivate::vcsCheckout(const QString &directory, const QByteA
 
 }
 
-bool SubversionPluginPrivate::managesDirectory(const QString &directory, QString *topLevel /* = 0 */) const
+bool SubversionPluginPrivate::managesDirectory(const FilePath &directory, FilePath *topLevel /* = 0 */) const
 {
-    const QDir dir(directory);
+    const QDir dir(directory.toString());
     if (topLevel)
         topLevel->clear();
 
@@ -1168,7 +1169,7 @@ bool SubversionPluginPrivate::managesDirectory(const QString &directory, QString
     while (!parentDir.isRoot()) {
         if (checkSVNSubDir(parentDir)) {
             if (topLevel)
-                *topLevel = parentDir.absolutePath();
+                *topLevel = FilePath::fromString(parentDir.absolutePath());
             return true;
         }
         if (!parentDir.cdUp())
@@ -1178,14 +1179,13 @@ bool SubversionPluginPrivate::managesDirectory(const QString &directory, QString
     return false;
 }
 
-bool SubversionPluginPrivate::managesFile(const QString &workingDirectory, const QString &fileName) const
+bool SubversionPluginPrivate::managesFile(const FilePath &workingDirectory, const QString &fileName) const
 {
     QStringList args;
     args << QLatin1String("status");
     args << SubversionClient::addAuthenticationOptions(m_settings)
          << QDir::toNativeSeparators(SubversionClient::escapeFile(fileName));
-    SubversionResponse response
-            = runSvn(workingDirectory, args, m_settings.timeout.value(), 0);
+    SubversionResponse response = runSvn(workingDirectory, args, m_settings.timeout.value(), 0);
     return response.stdOut.isEmpty() || response.stdOut.at(0) != QLatin1Char('?');
 }
 
@@ -1214,9 +1214,9 @@ Utils::Id SubversionPluginPrivate::id() const
     return Utils::Id(VcsBase::Constants::VCS_ID_SUBVERSION);
 }
 
-bool SubversionPluginPrivate::isVcsFileOrDirectory(const Utils::FilePath &fileName) const
+bool SubversionPluginPrivate::isVcsFileOrDirectory(const FilePath &filePath) const
 {
-    return isVcsDirectory(fileName);
+    return isVcsDirectory(filePath);
 }
 
 bool SubversionPluginPrivate::isConfigured() const
@@ -1246,40 +1246,37 @@ bool SubversionPluginPrivate::supportsOperation(Operation operation) const
     return rc;
 }
 
-bool SubversionPluginPrivate::vcsOpen(const QString & /* fileName */)
+bool SubversionPluginPrivate::vcsOpen(const FilePath & /* filePath */)
 {
     // Open for edit: N/A
     return true;
 }
 
-bool SubversionPluginPrivate::vcsAdd(const QString &fileName)
+bool SubversionPluginPrivate::vcsAdd(const FilePath &filePath)
 {
-    const QFileInfo fi(fileName);
-    return vcsAdd(fi.absolutePath(), fi.fileName());
+    return vcsAdd(filePath.parentDir(), filePath.fileName());
 }
 
-bool SubversionPluginPrivate::vcsDelete(const QString &fileName)
+bool SubversionPluginPrivate::vcsDelete(const FilePath &filePath)
 {
-    const QFileInfo fi(fileName);
-    return vcsDelete(fi.absolutePath(), fi.fileName());
+    return vcsDelete(filePath.parentDir(), filePath.fileName());
 }
 
-bool SubversionPluginPrivate::vcsMove(const QString &from, const QString &to)
+bool SubversionPluginPrivate::vcsMove(const FilePath &from, const FilePath &to)
 {
-    const QFileInfo fromInfo(from);
-    const QFileInfo toInfo(to);
-    return vcsMove(fromInfo.absolutePath(), fromInfo.absoluteFilePath(), toInfo.absoluteFilePath());
+    const QFileInfo fromInfo = from.toFileInfo();
+    const QFileInfo toInfo = to.toFileInfo();
+    return vcsMove(from.parentDir(), fromInfo.absoluteFilePath(), toInfo.absoluteFilePath());
 }
 
-bool SubversionPluginPrivate::vcsCreateRepository(const QString &)
+bool SubversionPluginPrivate::vcsCreateRepository(const FilePath &)
 {
     return false;
 }
 
-void SubversionPluginPrivate::vcsAnnotate(const QString &file, int line)
+void SubversionPluginPrivate::vcsAnnotate(const FilePath &filePath, int line)
 {
-    const QFileInfo fi(file);
-    vcsAnnotateHelper(fi.absolutePath(), fi.fileName(), QString(), line);
+    vcsAnnotateHelper(filePath.parentDir(), filePath.fileName(), QString(), line);
 }
 
 Core::ShellCommand *SubversionPluginPrivate::createInitialCheckoutCommand(const QString &url,
@@ -1293,17 +1290,17 @@ Core::ShellCommand *SubversionPluginPrivate::createInitialCheckoutCommand(const
     args << QLatin1String(Subversion::Constants::NON_INTERACTIVE_OPTION);
     args << extraArgs << url << localName;
 
-    auto command = new VcsBase::VcsCommand(baseDirectory.toString(), m_client->processEnvironment());
+    auto command = new VcsBase::VcsCommand(baseDirectory, m_client->processEnvironment());
     command->addJob({m_settings.binaryPath.filePath(), args}, -1);
     return command;
 }
 
-QString SubversionTopicCache::trackFile(const QString &repository)
+FilePath SubversionTopicCache::trackFile(const FilePath &repository)
 {
-    return m_plugin->monitorFile(repository);
+    return FilePath::fromString(m_plugin->monitorFile(repository));
 }
 
-QString SubversionTopicCache::refreshTopic(const QString &repository)
+QString SubversionTopicCache::refreshTopic(const FilePath &repository)
 {
     return m_plugin->synchronousTopic(repository);
 }
